searchNodes=[{"ref":"ExBreak.html","title":"ExBreak","type":"module","doc":"Provides circuit breaker functionality around function calls"},{"ref":"ExBreak.html#call/3","title":"ExBreak.call/3","type":"function","doc":"Call a function using a circuit breaker. The first option is the function to call, followed by an optional list of arguments. If the function succeeds, and does not return an error tuple (in the form {:error, any}), the value returned by the function will be returned. If the function fails (meaning it returns an {:error, any} tuple), a counter is incremented in a circuit breaker for that function. If the counter meets or exceeds the configured circuit breaker threshold, the breaker for that function is marked as tripped. If the function fails and the circuit breaker for that function has been marked as tripped (and has not expired), then {:error, :circuit_breaker_tripped} will be returned. A list of options can be provided as the final argument to call: timeout_sec (default 900) The number of seconds after which a tripped breaker times out and is re-opened threshold (default 10) The number of times an error is permitted before further calls will be ignored and will return an {:error, :circuilt_closed} match_exception (default fn _ -&gt; true end) A function called when an exception is raised during the function call. If it returns true, the breaker trip count is incremented. match_return (defaults to return true when matching {:error, _}) A function called on the return value of the function. If it returns true, the breaker trip count is incremented. Example This simple example increments the breaker when the return value is {:error, _}: iex&gt; ExBreak.call(fn ret -&gt; ret end, [:ok]) :ok iex&gt; fun = fn -&gt; {:error, :fail} end iex&gt; ExBreak.call(fun, [], threshold: 2) {:error, :fail} iex&gt; ExBreak.call(fun, [], threshold: 2) {:error, :fail} iex&gt; ExBreak.call(fun, [], threshold: 2) {:error, :circuit_breaker_tripped} In this example, we only increment when the return value is exactly {:error, :bad}: iex&gt; fun = fn ret -&gt; ret end iex&gt; opts = [threshold: 2, match_return: fn ...&gt; {:error, :bad} -&gt; true ...&gt; _ -&gt; false ...&gt; end] iex&gt; ExBreak.call(fun, [{:error, :not_bad}], opts) {:error, :not_bad} iex&gt; ExBreak.call(fun, [{:error, :not_bad}], opts) {:error, :not_bad} iex&gt; ExBreak.call(fun, [{:error, :not_bad}], opts) {:error, :not_bad} iex&gt; ExBreak.call(fun, [{:error, :bad}], opts) {:error, :bad} iex&gt; ExBreak.call(fun, [{:error, :bad}], opts) {:error, :bad} iex&gt; ExBreak.call(fun, [{:error, :bad}], opts) {:error, :circuit_breaker_tripped}"},{"ref":"ExBreak.html#t:opt/0","title":"ExBreak.opt/0","type":"type","doc":""},{"ref":"ExBreak.html#t:opts/0","title":"ExBreak.opts/0","type":"type","doc":""},{"ref":"ExBreak.Application.html","title":"ExBreak.Application","type":"module","doc":"The main ExBreak application, which starts the ExBreak.Supervisor"},{"ref":"ExBreak.Application.html#start/2","title":"ExBreak.Application.start/2","type":"function","doc":"Called when an application is started. This function is called when an application is started using Application.start/2 (and functions on top of that, such as Application.ensure_started/2). This function should start the top-level process of the application (which should be the top supervisor of the application&#39;s supervision tree if the application follows the OTP design principles around supervision). start_type defines how the application is started: :normal - used if the startup is a normal startup or if the application is distributed and is started on the current node because of a failover from another node and the application specification key :start_phases is :undefined. {:takeover, node} - used if the application is distributed and is started on the current node because of a failover on the node node. {:failover, node} - used if the application is distributed and is started on the current node because of a failover on node node, and the application specification key :start_phases is not :undefined. start_args are the arguments passed to the application in the :mod specification key (e.g., mod: {MyApp, [:my_args]}). This function should either return {:ok, pid} or {:ok, pid, state} if startup is successful. pid should be the PID of the top supervisor. state can be an arbitrary term, and if omitted will default to []; if the application is later stopped, state is passed to the stop/1 callback (see the documentation for the c:stop/1 callback for more information). use Application provides no default implementation for the start/2 callback. Callback implementation for Application.start/2."},{"ref":"ExBreak.Breaker.html","title":"ExBreak.Breaker","type":"module","doc":"A server that serves as a circuit breaker for a single function"},{"ref":"ExBreak.Breaker.html#child_spec/1","title":"ExBreak.Breaker.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"ExBreak.Breaker.html#increment/2","title":"ExBreak.Breaker.increment/2","type":"function","doc":"Increment a circuit breaker. If the new break count exceeds the given threshold, the breaker is also marked as tripped. iex&gt; {:ok, pid} = ExBreak.Breaker.start_link([]) iex&gt; ExBreak.Breaker.increment(pid, 10) iex&gt; ExBreak.Breaker.is_tripped(pid, 60) false"},{"ref":"ExBreak.Breaker.html#is_tripped/2","title":"ExBreak.Breaker.is_tripped/2","type":"function","doc":"Determine whether the given circuit breaker is tripped. The second argument, timeout_sec, is the time that must pass for a tripped circuit breaker to re-open. iex&gt; {:ok, pid} = ExBreak.Breaker.start_link([]) iex&gt; ExBreak.Breaker.increment(pid, 10) iex&gt; ExBreak.Breaker.is_tripped(pid, 10) false iex&gt; {:ok, pid} = ExBreak.Breaker.start_link([]) iex&gt; ExBreak.Breaker.increment(pid, 1) iex&gt; ExBreak.Breaker.is_tripped(pid, 10) true iex&gt; {:ok, pid} = ExBreak.Breaker.start_link([]) iex&gt; ExBreak.Breaker.increment(pid, 1) iex&gt; ExBreak.Breaker.is_tripped(pid, 0) false"},{"ref":"ExBreak.Breaker.html#new/0","title":"ExBreak.Breaker.new/0","type":"function","doc":"Create a new circuit breaker."},{"ref":"ExBreak.Breaker.html#reset_tripped/1","title":"ExBreak.Breaker.reset_tripped/1","type":"function","doc":"Reset a tripped circuit breaker by creating a new circuit breaker. If the circuit breaker is not tripped, it is simply returned. iex&gt; {:ok, pid} = ExBreak.Breaker.start_link([]) iex&gt; ExBreak.Breaker.increment(pid, 1) iex&gt; ExBreak.Breaker.is_tripped(pid, 10) true iex&gt; ExBreak.Breaker.reset_tripped(pid) iex&gt; ExBreak.Breaker.is_tripped(pid, 10) false"},{"ref":"ExBreak.Breaker.html#start_link/1","title":"ExBreak.Breaker.start_link/1","type":"function","doc":"Start a new breaker."},{"ref":"ExBreak.Breaker.html#t:t/0","title":"ExBreak.Breaker.t/0","type":"type","doc":"A struct representing the state of a circuit breaker break_count The number of breaks that have occurred tripped Whether the circuit breaker is tripped tripped_at The time at which the circuit breaker was tripped (or nil, if un-tripped)"},{"ref":"ExBreak.Registry.html","title":"ExBreak.Registry","type":"module","doc":"A registry of all running breakers"},{"ref":"ExBreak.Registry.html#child_spec/1","title":"ExBreak.Registry.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"ExBreak.Registry.html#find_breaker/1","title":"ExBreak.Registry.find_breaker/1","type":"function","doc":"Find a breaker, and return an error if it does not exist."},{"ref":"ExBreak.Registry.html#get_breaker/1","title":"ExBreak.Registry.get_breaker/1","type":"function","doc":"Look up a breaker, and create one if it does not exist. TODO: get_breaker/1 is a potential bottleneck, because it&#39;s the same GenServer call where we lookup all breakers. An ETS table isn&#39;t an option right now because we need get-or-create functionality."},{"ref":"ExBreak.Registry.html#start_link/1","title":"ExBreak.Registry.start_link/1","type":"function","doc":""},{"ref":"ExBreak.Supervisor.html","title":"ExBreak.Supervisor","type":"module","doc":""},{"ref":"ExBreak.Supervisor.html#child_spec/1","title":"ExBreak.Supervisor.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"ExBreak.Supervisor.html#init/1","title":"ExBreak.Supervisor.init/1","type":"function","doc":"Callback invoked to start the supervisor and during hot code upgrades. Developers typically invoke Supervisor.init/2 at the end of their init callback to return the proper supervision flags. Callback implementation for Supervisor.init/1."},{"ref":"ExBreak.Supervisor.html#start_link/1","title":"ExBreak.Supervisor.start_link/1","type":"function","doc":""},{"ref":"readme.html","title":"ExBreak","type":"extras","doc":"ExBreak ExBreak is a circuit breaker implementation for Elixir. When making function calls that may fail, you may find that you want to stop making those calls for a period of time after an error threshold is hit. This package provides a way to do that. defmodule GitHub do @moduledoc &quot;&quot;&quot; Makes calls to the GitHub API &quot;&quot;&quot; @base_url &quot;https://api.github.com&quot; @doc &quot;&quot;&quot; Make a GET request to the GitHub API. This request is wrapped in a circuit breaker—after 10 calls that return an error tuple (`{:error, any}`), the circuit breaker will trip, and the function will immediately return `{:error, :circuit_breaker_tripped}` for the next two minutes (120 seconds). &quot;&quot;&quot; def get(path, token: token) do ExBreak.call(&amp;do_get/2, [path, token], threshold: 10, timeout_sec: 120) end defp do_get(path, token) do HTTPoison.get(&quot;\#{@base_url}\#{path}&quot;, authorization: &quot;Bearer \#{token}&quot;) end end It is also possible to have more fine-grained control over when a circuit breaker&#39;s trip counter is incremented. For example, to ensure that only RuntimeErrors increment the counter, but not other exceptions, the match_exception option can be used. The match_exception option is a function that will be called with the exception. If it returns true, the trip counter will be incremented when an exception occurs. Otherwise, it will not. ExBreak.call( &amp;do_get/2, [path, token], threshold: 10, timeout_sec: 120, match_exception: fn %RuntimeError{} -&gt; true _ -&gt; false end ) Likewise, match_return can be used to designate what return values increment the trip counter. This option is a function that is called with the return value of the function passed to call/3. If it returns true, the trip counter will be incremented. Otherwise, it will not. ExBreak.call( &amp;do_get/2, [path, token], timeout_sec: 120, match_return: fn {:error, :service_unavailable} -&gt; true _ -&gt; false end )"},{"ref":"readme.html#installation","title":"ExBreak - Installation","type":"extras","doc":"If available in Hex, the package can be installed by adding ex_break to your list of dependencies in mix.exs: def deps do [ {:ex_break, &quot;~&gt; 0.1.0&quot;} ] end"},{"ref":"readme.html#architecture","title":"ExBreak - Architecture","type":"extras","doc":"When a call ExBreak.call/3 happens, the ExBreak module asks the ExBreak.Registry to get an ExBreak.Breaker state agent for the function passed to call/3. If an agent already exists, the registry returns it. Otherwise, it starts a new one using ExBreak.BreakerSupervisor and monitors it. If the breaker has already been tripped and the tripped state has not expired, the function passed to call/3 is never called and the value {:error, :circuit_breaker_tripped} is returned, instead. If, however, the breaker has not been tripped (or its tripped state has expired), the function passed to call/3 is called. If an exception is raised when calling the function, or if the return value of the function matches the pattern {:error, _}, a counter in the breaker&#39;s internal state is incremented, and either the exception is re-raised or the return value is returned. If the counter in the breaker&#39;s internal state meets a configured threshold, the breaker is marked as &quot;tripped&quot;, and subsequent calls to call/3 with the same function will return {:error, :circuit_breaker_tripped} immediately until the tripped state expires once again. ExBreak.Application This module is an Application which will start when you include ex_break in your application&#39;s dependencies. Its only responsibility is to start the ExBreak.Supervisor. ExBreak.Supervisor This is a Supervisor which starts ExBreak.DynamicSupervisor and ExBreak.Registry. ExBreak.BreakerSupervisor This module is a DynamicSupervisor that dynamically supervises ExBreak.Breaker agents on demand as they&#39;re needed. ExBreak.Registry This module is a registry of ExBreak.Breaker agents. When a call to ExBreak.call/3 happens, the registry finds or creates the ExBreak.Breaker registered for the given function call and returns it to the ExBreak module for use. When an ExBreak.Breaker process exits, it is de-registered. ExBreak.Breaker This module is an Agent which stores internal state about an individual circuit breaker. Architecture Diagram ╔═══════════════════════════╗ ║ ║░ ║ ExBreak.Application ║░ ║ ║░ ╚═══════════════════════════╝░ ░░░░░░░░░░░░░│░░░░░░░░░░░░░░░ │ ▼ ╔═══════════════════════════╗ ║ ║░ ║ ExBreak.Supervisor ║░ ║ ║░ ╚═══════════════════════════╝░ ░░░░░░░░░░░░░│░░░░░░░░░░░░░░░ │ ┌─────────────┴────────────────────────────┐ │ │ ▼ ▼ ╔═══════════════════════════╗ ╔═══════════════════════════╗ ║ ║░ ║ ║░ ║ ExBreak.BreakerSupervisor ║░ ║ ExBreak.Registry ║░ ║ ║░ ║ ║░ ╚═══════════════════════════╝░ ╚═══════════════════════════╝░ ░░░░░░░░░░░░░│░░░░░░░░░░░░░░░ ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░ │ ┌──────────┴───────────┬──────────────────────┐ │ │ │ ▼ ▼ ▼ ┌───────────────────┐ ┌───────────────────┐ ┌───────────────────┐ │ ExBreak.Breaker │ │ ExBreak.Breaker │ │ ExBreak.Breaker │ └───────────────────┘ └───────────────────┘ └───────────────────┘ Documentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc) and published on [HexDocs](https://hexdocs.pm). Once published, the docs can be found at [https://hexdocs.pm/ex_break](https://hexdocs.pm/ex_break)."},{"ref":"license.html","title":"LICENSE","type":"extras","doc":"Copyright (c) 2018 Jonathan Clem Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE."}]